#include <assert.h>

#include <aterm2.h>
#include <deprecated.h>
#include "builtins.h"

/*{{{  conversion functions */

ATerm stringToChars(const char *str)
{
  int len = strlen(str);
  int i;
  ATermList result = ATempty;

  for (i = len - 1; i >= 0; i--) {
    result = ATinsert(result, (ATerm) ATmakeInt(str[i]));
  }

  return (ATerm) result;
}

char *charsToString(ATerm arg)
{
  ATermList list = (ATermList) arg;
  int len = ATgetLength(list);
  int i;
  char *str;

  str = (char *) malloc(len+1);
  if (str == NULL) {
      return NULL;
  }

  for (i = 0; !ATisEmpty(list); list = ATgetNext(list), i++) {
    str[i] = (char) ATgetInt((ATermInt) ATgetFirst(list));
  }
  str[i] = '\0';

  return str;
}


/*}}}  */

/*{{{  typedefs */

typedef struct ATerm _D;
typedef struct ATerm _SepIntList;
typedef struct ATerm _SepDoubleList;
typedef struct ATerm _SepTermList;
typedef struct ATerm _SepStringList;
typedef struct ATerm _Lexical;

/*}}}  */

/*{{{  void initBuiltinsApi(void) */

void initBuiltinsApi(void)
{
  init_builtins_dict();
}

/*}}}  */

/*{{{  protect functions */

void protectD(D *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void protectSepIntList(SepIntList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void protectSepDoubleList(SepDoubleList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void protectSepTermList(SepTermList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void protectSepStringList(SepStringList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void protectLexical(Lexical *arg)
{
  ATprotect((ATerm*)((void*) arg));
}


/*}}}  */
/*{{{  term conversion functions */

/*{{{  D DFromTerm(ATerm t) */

D DFromTerm(ATerm t)
{
  return (D)t;
}

/*}}}  */
/*{{{  ATerm DToTerm(D arg) */

ATerm DToTerm(D arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SepIntList SepIntListFromTerm(ATerm t) */

SepIntList SepIntListFromTerm(ATerm t)
{
  return (SepIntList)t;
}

/*}}}  */
/*{{{  ATerm SepIntListToTerm(SepIntList arg) */

ATerm SepIntListToTerm(SepIntList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SepDoubleList SepDoubleListFromTerm(ATerm t) */

SepDoubleList SepDoubleListFromTerm(ATerm t)
{
  return (SepDoubleList)t;
}

/*}}}  */
/*{{{  ATerm SepDoubleListToTerm(SepDoubleList arg) */

ATerm SepDoubleListToTerm(SepDoubleList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SepTermList SepTermListFromTerm(ATerm t) */

SepTermList SepTermListFromTerm(ATerm t)
{
  return (SepTermList)t;
}

/*}}}  */
/*{{{  ATerm SepTermListToTerm(SepTermList arg) */

ATerm SepTermListToTerm(SepTermList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SepStringList SepStringListFromTerm(ATerm t) */

SepStringList SepStringListFromTerm(ATerm t)
{
  return (SepStringList)t;
}

/*}}}  */
/*{{{  ATerm SepStringListToTerm(SepStringList arg) */

ATerm SepStringListToTerm(SepStringList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  Lexical LexicalFromTerm(ATerm t) */

Lexical LexicalFromTerm(ATerm t)
{
  return (Lexical)t;
}

/*}}}  */
/*{{{  ATerm LexicalToTerm(Lexical arg) */

ATerm LexicalToTerm(Lexical arg)
{
  return (ATerm)arg;
}

/*}}}  */

/*}}}  */
/*{{{  list functions */

int getSepIntListLength (SepIntList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 3) + 1;
}
SepIntList reverseSepIntList(SepIntList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SepIntList) result;
}
SepIntList appendSepIntList(SepIntList arg0, char* sep, int arg1) {
  return concatSepIntList(arg0, sep, makeSepIntListSingle(arg1));
}
SepIntList concatSepIntList(SepIntList arg0, char* sep, SepIntList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = makeSepIntListMany((int)ATgetInt((ATermInt) ATgetFirst((ATermList) arg0)), sep,  arg1);
  arg1 = (SepIntList) ATgetNext((ATermList) arg1);
  return (SepIntList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SepIntList sliceSepIntList(SepIntList arg, int start, int end) {
  return (SepIntList) ATgetSlice((ATermList) arg, start * 3, end * 3);
}
int getSepIntListintAt(SepIntList arg, int index) {
 return (int)ATgetInt((ATermInt) ATelementAt((ATermList) arg,index * 3));
}
SepIntList replaceSepIntListintAt(SepIntList arg, int elem, int index) {
 return (SepIntList) ATreplace((ATermList) arg, (ATerm) ((ATerm) (ATerm) ATmakeInt(elem)), index * 3);
}
SepIntList makeSepIntList2(char* sep, int elem1, int elem2) {
  return makeSepIntListMany(elem1, sep, makeSepIntListSingle(elem2));
}
SepIntList makeSepIntList3(char* sep, int elem1, int elem2, int elem3) {
  return makeSepIntListMany(elem1, sep, makeSepIntList2(sep, elem2, elem3));
}
SepIntList makeSepIntList4(char* sep, int elem1, int elem2, int elem3, int elem4) {
  return makeSepIntListMany(elem1, sep, makeSepIntList3(sep, elem2, elem3, elem4));
}
SepIntList makeSepIntList5(char* sep, int elem1, int elem2, int elem3, int elem4, int elem5) {
  return makeSepIntListMany(elem1, sep, makeSepIntList4(sep, elem2, elem3, elem4, elem5));
}
SepIntList makeSepIntList6(char* sep, int elem1, int elem2, int elem3, int elem4, int elem5, int elem6) {
  return makeSepIntListMany(elem1, sep, makeSepIntList5(sep, elem2, elem3, elem4, elem5, elem6));
}
int getSepDoubleListLength (SepDoubleList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 3) + 1;
}
SepDoubleList reverseSepDoubleList(SepDoubleList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SepDoubleList) result;
}
SepDoubleList appendSepDoubleList(SepDoubleList arg0, char* sep, float arg1) {
  return concatSepDoubleList(arg0, sep, makeSepDoubleListSingle(arg1));
}
SepDoubleList concatSepDoubleList(SepDoubleList arg0, char* sep, SepDoubleList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = makeSepDoubleListMany((float)ATgetReal((ATermReal) ATgetFirst((ATermList) arg0)), sep,  arg1);
  arg1 = (SepDoubleList) ATgetNext((ATermList) arg1);
  return (SepDoubleList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SepDoubleList sliceSepDoubleList(SepDoubleList arg, int start, int end) {
  return (SepDoubleList) ATgetSlice((ATermList) arg, start * 3, end * 3);
}
float getSepDoubleListrealAt(SepDoubleList arg, int index) {
 return (float)ATgetReal((ATermReal) ATelementAt((ATermList) arg,index * 3));
}
SepDoubleList replaceSepDoubleListrealAt(SepDoubleList arg, float elem, int index) {
 return (SepDoubleList) ATreplace((ATermList) arg, (ATerm) ((ATerm) (ATerm) ATmakeReal(elem)), index * 3);
}
SepDoubleList makeSepDoubleList2(char* sep, float elem1, float elem2) {
  return makeSepDoubleListMany(elem1, sep, makeSepDoubleListSingle(elem2));
}
SepDoubleList makeSepDoubleList3(char* sep, float elem1, float elem2, float elem3) {
  return makeSepDoubleListMany(elem1, sep, makeSepDoubleList2(sep, elem2, elem3));
}
SepDoubleList makeSepDoubleList4(char* sep, float elem1, float elem2, float elem3, float elem4) {
  return makeSepDoubleListMany(elem1, sep, makeSepDoubleList3(sep, elem2, elem3, elem4));
}
SepDoubleList makeSepDoubleList5(char* sep, float elem1, float elem2, float elem3, float elem4, float elem5) {
  return makeSepDoubleListMany(elem1, sep, makeSepDoubleList4(sep, elem2, elem3, elem4, elem5));
}
SepDoubleList makeSepDoubleList6(char* sep, float elem1, float elem2, float elem3, float elem4, float elem5, float elem6) {
  return makeSepDoubleListMany(elem1, sep, makeSepDoubleList5(sep, elem2, elem3, elem4, elem5, elem6));
}
int getSepTermListLength (SepTermList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 3) + 1;
}
SepTermList reverseSepTermList(SepTermList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SepTermList) result;
}
SepTermList appendSepTermList(SepTermList arg0, char* sep, ATerm arg1) {
  return concatSepTermList(arg0, sep, makeSepTermListSingle(arg1));
}
SepTermList concatSepTermList(SepTermList arg0, char* sep, SepTermList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = makeSepTermListMany((ATerm)ATgetFirst((ATermList) arg0), sep,  arg1);
  arg1 = (SepTermList) ATgetNext((ATermList) arg1);
  return (SepTermList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SepTermList sliceSepTermList(SepTermList arg, int start, int end) {
  return (SepTermList) ATgetSlice((ATermList) arg, start * 3, end * 3);
}
ATerm getSepTermListtermAt(SepTermList arg, int index) {
 return (ATerm)ATelementAt((ATermList) arg,index * 3);
}
SepTermList replaceSepTermListtermAt(SepTermList arg, ATerm elem, int index) {
 return (SepTermList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 3);
}
SepTermList makeSepTermList2(char* sep, ATerm elem1, ATerm elem2) {
  return makeSepTermListMany(elem1, sep, makeSepTermListSingle(elem2));
}
SepTermList makeSepTermList3(char* sep, ATerm elem1, ATerm elem2, ATerm elem3) {
  return makeSepTermListMany(elem1, sep, makeSepTermList2(sep, elem2, elem3));
}
SepTermList makeSepTermList4(char* sep, ATerm elem1, ATerm elem2, ATerm elem3, ATerm elem4) {
  return makeSepTermListMany(elem1, sep, makeSepTermList3(sep, elem2, elem3, elem4));
}
SepTermList makeSepTermList5(char* sep, ATerm elem1, ATerm elem2, ATerm elem3, ATerm elem4, ATerm elem5) {
  return makeSepTermListMany(elem1, sep, makeSepTermList4(sep, elem2, elem3, elem4, elem5));
}
SepTermList makeSepTermList6(char* sep, ATerm elem1, ATerm elem2, ATerm elem3, ATerm elem4, ATerm elem5, ATerm elem6) {
  return makeSepTermListMany(elem1, sep, makeSepTermList5(sep, elem2, elem3, elem4, elem5, elem6));
}
int getSepStringListLength (SepStringList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 3) + 1;
}
SepStringList reverseSepStringList(SepStringList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SepStringList) result;
}
SepStringList appendSepStringList(SepStringList arg0, char* sep, char* arg1) {
  return concatSepStringList(arg0, sep, makeSepStringListSingle(arg1));
}
SepStringList concatSepStringList(SepStringList arg0, char* sep, SepStringList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = makeSepStringListMany((char*)ATgetName(ATgetAFun((ATermAppl) ATgetFirst((ATermList) arg0))), sep,  arg1);
  arg1 = (SepStringList) ATgetNext((ATermList) arg1);
  return (SepStringList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SepStringList sliceSepStringList(SepStringList arg, int start, int end) {
  return (SepStringList) ATgetSlice((ATermList) arg, start * 3, end * 3);
}
char* getSepStringListstrAt(SepStringList arg, int index) {
 return (char*)ATgetName(ATgetAFun((ATermAppl) ATelementAt((ATermList) arg,index * 3)));
}
SepStringList replaceSepStringListstrAt(SepStringList arg, char* elem, int index) {
 return (SepStringList) ATreplace((ATermList) arg, (ATerm) ((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(elem, 0, ATtrue))), index * 3);
}
SepStringList makeSepStringList2(char* sep, char* elem1, char* elem2) {
  return makeSepStringListMany(elem1, sep, makeSepStringListSingle(elem2));
}
SepStringList makeSepStringList3(char* sep, char* elem1, char* elem2, char* elem3) {
  return makeSepStringListMany(elem1, sep, makeSepStringList2(sep, elem2, elem3));
}
SepStringList makeSepStringList4(char* sep, char* elem1, char* elem2, char* elem3, char* elem4) {
  return makeSepStringListMany(elem1, sep, makeSepStringList3(sep, elem2, elem3, elem4));
}
SepStringList makeSepStringList5(char* sep, char* elem1, char* elem2, char* elem3, char* elem4, char* elem5) {
  return makeSepStringListMany(elem1, sep, makeSepStringList4(sep, elem2, elem3, elem4, elem5));
}
SepStringList makeSepStringList6(char* sep, char* elem1, char* elem2, char* elem3, char* elem4, char* elem5, char* elem6) {
  return makeSepStringListMany(elem1, sep, makeSepStringList5(sep, elem2, elem3, elem4, elem5, elem6));
}

/*}}}  */
/*{{{  constructors */

/*{{{  D makeDIinteger(int integer) */

D makeDIinteger(int integer)
{
  return (D)(ATerm)ATmakeAppl1(afun0, (ATerm) (ATerm) ATmakeInt(integer));
}

/*}}}  */
/*{{{  D makeDDdouble(float number) */

D makeDDdouble(float number)
{
  return (D)(ATerm)ATmakeAppl1(afun1, (ATerm) (ATerm) ATmakeReal(number));
}

/*}}}  */
/*{{{  D makeDTrm(ATerm trm) */

D makeDTrm(ATerm trm)
{
  return (D)(ATerm)ATmakeAppl1(afun2, (ATerm) trm);
}

/*}}}  */
/*{{{  D makeDLst(ATermList lst) */

D makeDLst(ATermList lst)
{
  return (D)(ATerm)ATmakeAppl1(afun3, (ATerm) (ATerm) lst);
}

/*}}}  */
/*{{{  D makeDSstring(const char* string) */

D makeDSstring(const char* string)
{
  return (D)(ATerm)ATmakeAppl1(afun4, (ATerm) (ATerm) ATmakeAppl(ATmakeAFun(string, 0, ATtrue)));
}

/*}}}  */
/*{{{  D makeDAll(int integer, float number, ATerm trm, const char* string) */

D makeDAll(int integer, float number, ATerm trm, const char* string)
{
  return (D)(ATerm)ATmakeAppl4(afun5, (ATerm) (ATerm) ATmakeInt(integer), (ATerm) (ATerm) ATmakeReal(number), (ATerm) trm, (ATerm) (ATerm) ATmakeAppl(ATmakeAFun(string, 0, ATtrue)));
}

/*}}}  */
/*{{{  D makeDSome(int integer, float number) */

D makeDSome(int integer, float number)
{
  return (D)(ATerm)ATmakeAppl2(afun6, (ATerm) (ATerm) ATmakeInt(integer), (ATerm) (ATerm) ATmakeReal(number));
}

/*}}}  */
/*{{{  SepIntList makeSepIntListEmpty(void) */

SepIntList makeSepIntListEmpty(void)
{
  return (SepIntList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SepIntList makeSepIntListSingle(int head) */

SepIntList makeSepIntListSingle(int head)
{
  return (SepIntList)(ATerm)ATmakeList1((ATerm) (ATerm) ATmakeInt(head));
}

/*}}}  */
/*{{{  SepIntList makeSepIntListMany(int head, const char* sep, SepIntList tail) */

SepIntList makeSepIntListMany(int head, const char* sep, SepIntList tail)
{
  if (isSepIntListEmpty(tail)) {
    return makeSepIntListSingle(head);
  }
  return (SepIntList)(ATerm)ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm)ATmakeAppl0(afun7)), (ATerm) (ATerm) ATmakeAppl(ATmakeAFun(sep, 0, ATtrue))), (ATerm) (ATerm) ATmakeInt(head));
}

/*}}}  */
/*{{{  SepDoubleList makeSepDoubleListEmpty(void) */

SepDoubleList makeSepDoubleListEmpty(void)
{
  return (SepDoubleList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SepDoubleList makeSepDoubleListSingle(float head) */

SepDoubleList makeSepDoubleListSingle(float head)
{
  return (SepDoubleList)(ATerm)ATmakeList1((ATerm) (ATerm) ATmakeReal(head));
}

/*}}}  */
/*{{{  SepDoubleList makeSepDoubleListMany(float head, const char* sep, SepDoubleList tail) */

SepDoubleList makeSepDoubleListMany(float head, const char* sep, SepDoubleList tail)
{
  if (isSepDoubleListEmpty(tail)) {
    return makeSepDoubleListSingle(head);
  }
  return (SepDoubleList)(ATerm)ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm)ATmakeAppl0(afun7)), (ATerm) (ATerm) ATmakeAppl(ATmakeAFun(sep, 0, ATtrue))), (ATerm) (ATerm) ATmakeReal(head));
}

/*}}}  */
/*{{{  SepTermList makeSepTermListEmpty(void) */

SepTermList makeSepTermListEmpty(void)
{
  return (SepTermList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SepTermList makeSepTermListSingle(ATerm head) */

SepTermList makeSepTermListSingle(ATerm head)
{
  return (SepTermList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SepTermList makeSepTermListMany(ATerm head, const char* sep, SepTermList tail) */

SepTermList makeSepTermListMany(ATerm head, const char* sep, SepTermList tail)
{
  if (isSepTermListEmpty(tail)) {
    return makeSepTermListSingle(head);
  }
  return (SepTermList)(ATerm)ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm)ATmakeAppl0(afun7)), (ATerm) (ATerm) ATmakeAppl(ATmakeAFun(sep, 0, ATtrue))), (ATerm) head);
}

/*}}}  */
/*{{{  SepStringList makeSepStringListEmpty(void) */

SepStringList makeSepStringListEmpty(void)
{
  return (SepStringList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SepStringList makeSepStringListSingle(const char* head) */

SepStringList makeSepStringListSingle(const char* head)
{
  return (SepStringList)(ATerm)ATmakeList1((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(head, 0, ATtrue)));
}

/*}}}  */
/*{{{  SepStringList makeSepStringListMany(const char* head, const char* sep, SepStringList tail) */

SepStringList makeSepStringListMany(const char* head, const char* sep, SepStringList tail)
{
  if (isSepStringListEmpty(tail)) {
    return makeSepStringListSingle(head);
  }
  return (SepStringList)(ATerm)ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm)ATmakeAppl0(afun7)), (ATerm) (ATerm) ATmakeAppl(ATmakeAFun(sep, 0, ATtrue))), (ATerm) (ATerm) ATmakeAppl(ATmakeAFun(head, 0, ATtrue)));
}

/*}}}  */
/*{{{  Lexical makeLexicalDefault(const char* string) */

Lexical makeLexicalDefault(const char* string)
{
  return (Lexical)(ATerm)ATmakeAppl1(afun8, (ATerm) ((ATerm) stringToChars(string)));
}

/*}}}  */

/*}}}  */
/*{{{  equality functions */

ATbool isEqualD(D arg0, D arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualSepIntList(SepIntList arg0, SepIntList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualSepDoubleList(SepDoubleList arg0, SepDoubleList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualSepTermList(SepTermList arg0, SepTermList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualSepStringList(SepStringList arg0, SepStringList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualLexical(Lexical arg0, Lexical arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

/*}}}  */
/*{{{  D accessors */

/*{{{  ATbool isValidD(D arg) */

ATbool isValidD(D arg)
{
  if (isDIinteger(arg)) {
    return ATtrue;
  }
  else if (isDDdouble(arg)) {
    return ATtrue;
  }
  else if (isDTrm(arg)) {
    return ATtrue;
  }
  else if (isDLst(arg)) {
    return ATtrue;
  }
  else if (isDSstring(arg)) {
    return ATtrue;
  }
  else if (isDAll(arg)) {
    return ATtrue;
  }
  else if (isDSome(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isDIinteger(D arg) */

inline ATbool isDIinteger(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDIinteger, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isDDdouble(D arg) */

inline ATbool isDDdouble(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDDdouble, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isDTrm(D arg) */

inline ATbool isDTrm(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDTrm, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isDLst(D arg) */

inline ATbool isDLst(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDLst, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isDSstring(D arg) */

inline ATbool isDSstring(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDSstring, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isDAll(D arg) */

inline ATbool isDAll(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDAll, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isDSome(D arg) */

inline ATbool isDSome(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDSome, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool hasDInteger(D arg) */

ATbool hasDInteger(D arg)
{
  if (isDIinteger(arg)) {
    return ATtrue;
  }
  else if (isDAll(arg)) {
    return ATtrue;
  }
  else if (isDSome(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasDNumber(D arg) */

ATbool hasDNumber(D arg)
{
  if (isDDdouble(arg)) {
    return ATtrue;
  }
  else if (isDAll(arg)) {
    return ATtrue;
  }
  else if (isDSome(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasDTrm(D arg) */

ATbool hasDTrm(D arg)
{
  if (isDTrm(arg)) {
    return ATtrue;
  }
  else if (isDAll(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasDLst(D arg) */

ATbool hasDLst(D arg)
{
  if (isDLst(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasDString(D arg) */

ATbool hasDString(D arg)
{
  if (isDSstring(arg)) {
    return ATtrue;
  }
  else if (isDAll(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  int getDInteger(D arg) */

int getDInteger(D arg)
{
  if (isDIinteger(arg)) {
    return (int)ATgetInt((ATermInt) ATgetArgument((ATermAppl)arg, 0));
  }
  else if (isDAll(arg)) {
    return (int)ATgetInt((ATermInt) ATgetArgument((ATermAppl)arg, 0));
  }
  else 
    return (int)ATgetInt((ATermInt) ATgetArgument((ATermAppl)arg, 0));
}

/*}}}  */
/*{{{  float getDNumber(D arg) */

float getDNumber(D arg)
{
  if (isDDdouble(arg)) {
    return (float)ATgetReal((ATermReal) ATgetArgument((ATermAppl)arg, 0));
  }
  else if (isDAll(arg)) {
    return (float)ATgetReal((ATermReal) ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (float)ATgetReal((ATermReal) ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  ATerm getDTrm(D arg) */

ATerm getDTrm(D arg)
{
  if (isDTrm(arg)) {
    return (ATerm)ATgetArgument((ATermAppl)arg, 0);
  }
  else 
    return (ATerm)ATgetArgument((ATermAppl)arg, 2);
}

/*}}}  */
/*{{{  ATermList getDLst(D arg) */

ATermList getDLst(D arg)
{
  
    return (ATermList)(ATermList) ATgetArgument((ATermAppl)arg, 0);
}

/*}}}  */
/*{{{  char* getDString(D arg) */

char* getDString(D arg)
{
  if (isDSstring(arg)) {
    return (char*)ATgetName(ATgetAFun((ATermAppl) ATgetArgument((ATermAppl)arg, 0)));
  }
  else 
    return (char*)ATgetName(ATgetAFun((ATermAppl) ATgetArgument((ATermAppl)arg, 3)));
}

/*}}}  */
/*{{{  D setDInteger(D arg, int integer) */

D setDInteger(D arg, int integer)
{
  if (isDIinteger(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(integer)), 0);
  }
  else if (isDAll(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(integer)), 0);
  }
  else if (isDSome(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(integer)), 0);
  }

  ATabort("D has no Integer: %t\n", arg);
  return (D)NULL;
}

/*}}}  */
/*{{{  D setDNumber(D arg, float number) */

D setDNumber(D arg, float number)
{
  if (isDDdouble(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeReal(number)), 0);
  }
  else if (isDAll(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeReal(number)), 1);
  }
  else if (isDSome(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeReal(number)), 1);
  }

  ATabort("D has no Number: %t\n", arg);
  return (D)NULL;
}

/*}}}  */
/*{{{  D setDTrm(D arg, ATerm trm) */

D setDTrm(D arg, ATerm trm)
{
  if (isDTrm(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) trm), 0);
  }
  else if (isDAll(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) trm), 2);
  }

  ATabort("D has no Trm: %t\n", arg);
  return (D)NULL;
}

/*}}}  */
/*{{{  D setDLst(D arg, ATermList lst) */

D setDLst(D arg, ATermList lst)
{
  if (isDLst(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) lst), 0);
  }

  ATabort("D has no Lst: %t\n", arg);
  return (D)NULL;
}

/*}}}  */
/*{{{  D setDString(D arg, const char* string) */

D setDString(D arg, const char* string)
{
  if (isDSstring(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(string, 0, ATtrue))), 0);
  }
  else if (isDAll(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(string, 0, ATtrue))), 3);
  }

  ATabort("D has no String: %t\n", arg);
  return (D)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SepIntList accessors */

/*{{{  ATbool isValidSepIntList(SepIntList arg) */

ATbool isValidSepIntList(SepIntList arg)
{
  if (isSepIntListEmpty(arg)) {
    return ATtrue;
  }
  else if (isSepIntListSingle(arg)) {
    return ATtrue;
  }
  else if (isSepIntListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isSepIntListEmpty(SepIntList arg) */

inline ATbool isSepIntListEmpty(SepIntList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternSepIntListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool isSepIntListSingle(SepIntList arg) */

inline ATbool isSepIntListSingle(SepIntList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternSepIntListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isSepIntListMany(SepIntList arg) */

inline ATbool isSepIntListMany(SepIntList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternSepIntListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool hasSepIntListHead(SepIntList arg) */

ATbool hasSepIntListHead(SepIntList arg)
{
  if (isSepIntListSingle(arg)) {
    return ATtrue;
  }
  else if (isSepIntListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasSepIntListSep(SepIntList arg) */

ATbool hasSepIntListSep(SepIntList arg)
{
  if (isSepIntListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasSepIntListTail(SepIntList arg) */

ATbool hasSepIntListTail(SepIntList arg)
{
  if (isSepIntListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SepIntList getSepIntListTail(SepIntList arg) */

SepIntList getSepIntListTail(SepIntList arg)
{
  assert(!isSepIntListEmpty(arg) && "getTail on an empty list");
  if (isSepIntListSingle(arg)) {
    return (SepIntList) makeSepIntListEmpty();
  }
  else {
  
    return (SepIntList)ATgetTail((ATermList)arg, 3);
  }
}

/*}}}  */
/*{{{  int getSepIntListHead(SepIntList arg) */

int getSepIntListHead(SepIntList arg)
{
  if (isSepIntListSingle(arg)) {
    return (int)ATgetInt((ATermInt) ATgetFirst((ATermList)arg));
  }
  else 
    return (int)ATgetInt((ATermInt) ATgetFirst((ATermList)arg));
}

/*}}}  */
/*{{{  char* getSepIntListSep(SepIntList arg) */

char* getSepIntListSep(SepIntList arg)
{
  
    return (char*)ATgetName(ATgetAFun((ATermAppl) ATelementAt((ATermList)arg, 1)));
}

/*}}}  */
/*{{{  SepIntList setSepIntListHead(SepIntList arg, int head) */

SepIntList setSepIntListHead(SepIntList arg, int head)
{
  if (isSepIntListSingle(arg)) {
    return (SepIntList)ATreplace((ATermList)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(head)), 0);
  }
  else if (isSepIntListMany(arg)) {
    return (SepIntList)ATreplace((ATermList)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(head)), 0);
  }

  ATabort("SepIntList has no Head: %t\n", arg);
  return (SepIntList)NULL;
}

/*}}}  */
/*{{{  SepIntList setSepIntListSep(SepIntList arg, const char* sep) */

SepIntList setSepIntListSep(SepIntList arg, const char* sep)
{
  if (isSepIntListMany(arg)) {
    return (SepIntList)ATreplace((ATermList)arg, (ATerm)((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(sep, 0, ATtrue))), 1);
  }

  ATabort("SepIntList has no Sep: %t\n", arg);
  return (SepIntList)NULL;
}

/*}}}  */
/*{{{  SepIntList setSepIntListTail(SepIntList arg, SepIntList tail) */

SepIntList setSepIntListTail(SepIntList arg, SepIntList tail)
{
  if (isSepIntListMany(arg)) {
    return (SepIntList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 3);
  }

  ATabort("SepIntList has no Tail: %t\n", arg);
  return (SepIntList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SepDoubleList accessors */

/*{{{  ATbool isValidSepDoubleList(SepDoubleList arg) */

ATbool isValidSepDoubleList(SepDoubleList arg)
{
  if (isSepDoubleListEmpty(arg)) {
    return ATtrue;
  }
  else if (isSepDoubleListSingle(arg)) {
    return ATtrue;
  }
  else if (isSepDoubleListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isSepDoubleListEmpty(SepDoubleList arg) */

inline ATbool isSepDoubleListEmpty(SepDoubleList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternSepDoubleListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool isSepDoubleListSingle(SepDoubleList arg) */

inline ATbool isSepDoubleListSingle(SepDoubleList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternSepDoubleListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isSepDoubleListMany(SepDoubleList arg) */

inline ATbool isSepDoubleListMany(SepDoubleList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternSepDoubleListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool hasSepDoubleListHead(SepDoubleList arg) */

ATbool hasSepDoubleListHead(SepDoubleList arg)
{
  if (isSepDoubleListSingle(arg)) {
    return ATtrue;
  }
  else if (isSepDoubleListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasSepDoubleListSep(SepDoubleList arg) */

ATbool hasSepDoubleListSep(SepDoubleList arg)
{
  if (isSepDoubleListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasSepDoubleListTail(SepDoubleList arg) */

ATbool hasSepDoubleListTail(SepDoubleList arg)
{
  if (isSepDoubleListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SepDoubleList getSepDoubleListTail(SepDoubleList arg) */

SepDoubleList getSepDoubleListTail(SepDoubleList arg)
{
  assert(!isSepDoubleListEmpty(arg) && "getTail on an empty list");
  if (isSepDoubleListSingle(arg)) {
    return (SepDoubleList) makeSepDoubleListEmpty();
  }
  else {
  
    return (SepDoubleList)ATgetTail((ATermList)arg, 3);
  }
}

/*}}}  */
/*{{{  float getSepDoubleListHead(SepDoubleList arg) */

float getSepDoubleListHead(SepDoubleList arg)
{
  if (isSepDoubleListSingle(arg)) {
    return (float)ATgetReal((ATermReal) ATgetFirst((ATermList)arg));
  }
  else 
    return (float)ATgetReal((ATermReal) ATgetFirst((ATermList)arg));
}

/*}}}  */
/*{{{  char* getSepDoubleListSep(SepDoubleList arg) */

char* getSepDoubleListSep(SepDoubleList arg)
{
  
    return (char*)ATgetName(ATgetAFun((ATermAppl) ATelementAt((ATermList)arg, 1)));
}

/*}}}  */
/*{{{  SepDoubleList setSepDoubleListHead(SepDoubleList arg, float head) */

SepDoubleList setSepDoubleListHead(SepDoubleList arg, float head)
{
  if (isSepDoubleListSingle(arg)) {
    return (SepDoubleList)ATreplace((ATermList)arg, (ATerm)((ATerm) (ATerm) ATmakeReal(head)), 0);
  }
  else if (isSepDoubleListMany(arg)) {
    return (SepDoubleList)ATreplace((ATermList)arg, (ATerm)((ATerm) (ATerm) ATmakeReal(head)), 0);
  }

  ATabort("SepDoubleList has no Head: %t\n", arg);
  return (SepDoubleList)NULL;
}

/*}}}  */
/*{{{  SepDoubleList setSepDoubleListSep(SepDoubleList arg, const char* sep) */

SepDoubleList setSepDoubleListSep(SepDoubleList arg, const char* sep)
{
  if (isSepDoubleListMany(arg)) {
    return (SepDoubleList)ATreplace((ATermList)arg, (ATerm)((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(sep, 0, ATtrue))), 1);
  }

  ATabort("SepDoubleList has no Sep: %t\n", arg);
  return (SepDoubleList)NULL;
}

/*}}}  */
/*{{{  SepDoubleList setSepDoubleListTail(SepDoubleList arg, SepDoubleList tail) */

SepDoubleList setSepDoubleListTail(SepDoubleList arg, SepDoubleList tail)
{
  if (isSepDoubleListMany(arg)) {
    return (SepDoubleList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 3);
  }

  ATabort("SepDoubleList has no Tail: %t\n", arg);
  return (SepDoubleList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SepTermList accessors */

/*{{{  ATbool isValidSepTermList(SepTermList arg) */

ATbool isValidSepTermList(SepTermList arg)
{
  if (isSepTermListEmpty(arg)) {
    return ATtrue;
  }
  else if (isSepTermListSingle(arg)) {
    return ATtrue;
  }
  else if (isSepTermListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isSepTermListEmpty(SepTermList arg) */

inline ATbool isSepTermListEmpty(SepTermList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternSepTermListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool isSepTermListSingle(SepTermList arg) */

inline ATbool isSepTermListSingle(SepTermList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternSepTermListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isSepTermListMany(SepTermList arg) */

inline ATbool isSepTermListMany(SepTermList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternSepTermListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool hasSepTermListHead(SepTermList arg) */

ATbool hasSepTermListHead(SepTermList arg)
{
  if (isSepTermListSingle(arg)) {
    return ATtrue;
  }
  else if (isSepTermListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasSepTermListSep(SepTermList arg) */

ATbool hasSepTermListSep(SepTermList arg)
{
  if (isSepTermListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasSepTermListTail(SepTermList arg) */

ATbool hasSepTermListTail(SepTermList arg)
{
  if (isSepTermListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SepTermList getSepTermListTail(SepTermList arg) */

SepTermList getSepTermListTail(SepTermList arg)
{
  assert(!isSepTermListEmpty(arg) && "getTail on an empty list");
  if (isSepTermListSingle(arg)) {
    return (SepTermList) makeSepTermListEmpty();
  }
  else {
  
    return (SepTermList)ATgetTail((ATermList)arg, 3);
  }
}

/*}}}  */
/*{{{  ATerm getSepTermListHead(SepTermList arg) */

ATerm getSepTermListHead(SepTermList arg)
{
  if (isSepTermListSingle(arg)) {
    return (ATerm)ATgetFirst((ATermList)arg);
  }
  else 
    return (ATerm)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  char* getSepTermListSep(SepTermList arg) */

char* getSepTermListSep(SepTermList arg)
{
  
    return (char*)ATgetName(ATgetAFun((ATermAppl) ATelementAt((ATermList)arg, 1)));
}

/*}}}  */
/*{{{  SepTermList setSepTermListHead(SepTermList arg, ATerm head) */

SepTermList setSepTermListHead(SepTermList arg, ATerm head)
{
  if (isSepTermListSingle(arg)) {
    return (SepTermList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (isSepTermListMany(arg)) {
    return (SepTermList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("SepTermList has no Head: %t\n", arg);
  return (SepTermList)NULL;
}

/*}}}  */
/*{{{  SepTermList setSepTermListSep(SepTermList arg, const char* sep) */

SepTermList setSepTermListSep(SepTermList arg, const char* sep)
{
  if (isSepTermListMany(arg)) {
    return (SepTermList)ATreplace((ATermList)arg, (ATerm)((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(sep, 0, ATtrue))), 1);
  }

  ATabort("SepTermList has no Sep: %t\n", arg);
  return (SepTermList)NULL;
}

/*}}}  */
/*{{{  SepTermList setSepTermListTail(SepTermList arg, SepTermList tail) */

SepTermList setSepTermListTail(SepTermList arg, SepTermList tail)
{
  if (isSepTermListMany(arg)) {
    return (SepTermList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 3);
  }

  ATabort("SepTermList has no Tail: %t\n", arg);
  return (SepTermList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SepStringList accessors */

/*{{{  ATbool isValidSepStringList(SepStringList arg) */

ATbool isValidSepStringList(SepStringList arg)
{
  if (isSepStringListEmpty(arg)) {
    return ATtrue;
  }
  else if (isSepStringListSingle(arg)) {
    return ATtrue;
  }
  else if (isSepStringListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isSepStringListEmpty(SepStringList arg) */

inline ATbool isSepStringListEmpty(SepStringList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternSepStringListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool isSepStringListSingle(SepStringList arg) */

inline ATbool isSepStringListSingle(SepStringList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternSepStringListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isSepStringListMany(SepStringList arg) */

inline ATbool isSepStringListMany(SepStringList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternSepStringListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool hasSepStringListHead(SepStringList arg) */

ATbool hasSepStringListHead(SepStringList arg)
{
  if (isSepStringListSingle(arg)) {
    return ATtrue;
  }
  else if (isSepStringListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasSepStringListSep(SepStringList arg) */

ATbool hasSepStringListSep(SepStringList arg)
{
  if (isSepStringListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool hasSepStringListTail(SepStringList arg) */

ATbool hasSepStringListTail(SepStringList arg)
{
  if (isSepStringListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SepStringList getSepStringListTail(SepStringList arg) */

SepStringList getSepStringListTail(SepStringList arg)
{
  assert(!isSepStringListEmpty(arg) && "getTail on an empty list");
  if (isSepStringListSingle(arg)) {
    return (SepStringList) makeSepStringListEmpty();
  }
  else {
  
    return (SepStringList)ATgetTail((ATermList)arg, 3);
  }
}

/*}}}  */
/*{{{  char* getSepStringListHead(SepStringList arg) */

char* getSepStringListHead(SepStringList arg)
{
  if (isSepStringListSingle(arg)) {
    return (char*)ATgetName(ATgetAFun((ATermAppl) ATgetFirst((ATermList)arg)));
  }
  else 
    return (char*)ATgetName(ATgetAFun((ATermAppl) ATgetFirst((ATermList)arg)));
}

/*}}}  */
/*{{{  char* getSepStringListSep(SepStringList arg) */

char* getSepStringListSep(SepStringList arg)
{
  
    return (char*)ATgetName(ATgetAFun((ATermAppl) ATelementAt((ATermList)arg, 1)));
}

/*}}}  */
/*{{{  SepStringList setSepStringListHead(SepStringList arg, const char* head) */

SepStringList setSepStringListHead(SepStringList arg, const char* head)
{
  if (isSepStringListSingle(arg)) {
    return (SepStringList)ATreplace((ATermList)arg, (ATerm)((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(head, 0, ATtrue))), 0);
  }
  else if (isSepStringListMany(arg)) {
    return (SepStringList)ATreplace((ATermList)arg, (ATerm)((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(head, 0, ATtrue))), 0);
  }

  ATabort("SepStringList has no Head: %t\n", arg);
  return (SepStringList)NULL;
}

/*}}}  */
/*{{{  SepStringList setSepStringListSep(SepStringList arg, const char* sep) */

SepStringList setSepStringListSep(SepStringList arg, const char* sep)
{
  if (isSepStringListMany(arg)) {
    return (SepStringList)ATreplace((ATermList)arg, (ATerm)((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(sep, 0, ATtrue))), 1);
  }

  ATabort("SepStringList has no Sep: %t\n", arg);
  return (SepStringList)NULL;
}

/*}}}  */
/*{{{  SepStringList setSepStringListTail(SepStringList arg, SepStringList tail) */

SepStringList setSepStringListTail(SepStringList arg, SepStringList tail)
{
  if (isSepStringListMany(arg)) {
    return (SepStringList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 3);
  }

  ATabort("SepStringList has no Tail: %t\n", arg);
  return (SepStringList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  Lexical accessors */

/*{{{  ATbool isValidLexical(Lexical arg) */

ATbool isValidLexical(Lexical arg)
{
  if (isLexicalDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isLexicalDefault(Lexical arg) */

inline ATbool isLexicalDefault(Lexical arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternLexicalDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool hasLexicalString(Lexical arg) */

ATbool hasLexicalString(Lexical arg)
{
  if (isLexicalDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char* getLexicalString(Lexical arg) */

char* getLexicalString(Lexical arg)
{
  
    return (char*)charsToString((ATerm)ATgetArgument((ATermAppl)arg, 0));
}

/*}}}  */
/*{{{  Lexical setLexicalString(Lexical arg, const char* string) */

Lexical setLexicalString(Lexical arg, const char* string)
{
  if (isLexicalDefault(arg)) {
    return (Lexical)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) ((ATerm) stringToChars(string))), 0);
  }

  ATabort("Lexical has no String: %t\n", arg);
  return (Lexical)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  sort visitors */

/*{{{  D visitD(D arg, int (*acceptInteger)(int), float (*acceptNumber)(float), ATerm (*acceptTrm)(ATerm), ATermList (*acceptLst)(ATermList), char* (*acceptString)(char*)) */

D visitD(D arg, int (*acceptInteger)(int), float (*acceptNumber)(float), ATerm (*acceptTrm)(ATerm), ATermList (*acceptLst)(ATermList), char* (*acceptString)(char*))
{
  if (isDIinteger(arg)) {
    return makeDIinteger(
        acceptInteger ? acceptInteger(getDInteger(arg)) : getDInteger(arg));
  }
  if (isDDdouble(arg)) {
    return makeDDdouble(
        acceptNumber ? acceptNumber(getDNumber(arg)) : getDNumber(arg));
  }
  if (isDTrm(arg)) {
    return makeDTrm(
        acceptTrm ? acceptTrm(getDTrm(arg)) : getDTrm(arg));
  }
  if (isDLst(arg)) {
    return makeDLst(
        acceptLst ? acceptLst(getDLst(arg)) : getDLst(arg));
  }
  if (isDSstring(arg)) {
    return makeDSstring(
        acceptString ? acceptString(getDString(arg)) : getDString(arg));
  }
  if (isDAll(arg)) {
    return makeDAll(
        acceptInteger ? acceptInteger(getDInteger(arg)) : getDInteger(arg),
        acceptNumber ? acceptNumber(getDNumber(arg)) : getDNumber(arg),
        acceptTrm ? acceptTrm(getDTrm(arg)) : getDTrm(arg),
        acceptString ? acceptString(getDString(arg)) : getDString(arg));
  }
  if (isDSome(arg)) {
    return makeDSome(
        acceptInteger ? acceptInteger(getDInteger(arg)) : getDInteger(arg),
        acceptNumber ? acceptNumber(getDNumber(arg)) : getDNumber(arg));
  }
  ATabort("not a D: %t\n", arg);
  return (D)NULL;
}

/*}}}  */
/*{{{  SepIntList visitSepIntList(SepIntList arg, int (*acceptHead)(int), char* (*acceptSep)(char*)) */

SepIntList visitSepIntList(SepIntList arg, int (*acceptHead)(int), char* (*acceptSep)(char*))
{
  if (isSepIntListEmpty(arg)) {
    return makeSepIntListEmpty();
  }
  if (isSepIntListSingle(arg)) {
    return makeSepIntListSingle(
        acceptHead ? acceptHead(getSepIntListHead(arg)) : getSepIntListHead(arg));
  }
  if (isSepIntListMany(arg)) {
    return makeSepIntListMany(
        acceptHead ? acceptHead(getSepIntListHead(arg)) : getSepIntListHead(arg),
        acceptSep ? acceptSep(getSepIntListSep(arg)) : getSepIntListSep(arg),
        visitSepIntList(getSepIntListTail(arg), acceptHead, acceptSep));
  }
  ATabort("not a SepIntList: %t\n", arg);
  return (SepIntList)NULL;
}

/*}}}  */
/*{{{  SepDoubleList visitSepDoubleList(SepDoubleList arg, float (*acceptHead)(float), char* (*acceptSep)(char*)) */

SepDoubleList visitSepDoubleList(SepDoubleList arg, float (*acceptHead)(float), char* (*acceptSep)(char*))
{
  if (isSepDoubleListEmpty(arg)) {
    return makeSepDoubleListEmpty();
  }
  if (isSepDoubleListSingle(arg)) {
    return makeSepDoubleListSingle(
        acceptHead ? acceptHead(getSepDoubleListHead(arg)) : getSepDoubleListHead(arg));
  }
  if (isSepDoubleListMany(arg)) {
    return makeSepDoubleListMany(
        acceptHead ? acceptHead(getSepDoubleListHead(arg)) : getSepDoubleListHead(arg),
        acceptSep ? acceptSep(getSepDoubleListSep(arg)) : getSepDoubleListSep(arg),
        visitSepDoubleList(getSepDoubleListTail(arg), acceptHead, acceptSep));
  }
  ATabort("not a SepDoubleList: %t\n", arg);
  return (SepDoubleList)NULL;
}

/*}}}  */
/*{{{  SepTermList visitSepTermList(SepTermList arg, ATerm (*acceptHead)(ATerm), char* (*acceptSep)(char*)) */

SepTermList visitSepTermList(SepTermList arg, ATerm (*acceptHead)(ATerm), char* (*acceptSep)(char*))
{
  if (isSepTermListEmpty(arg)) {
    return makeSepTermListEmpty();
  }
  if (isSepTermListSingle(arg)) {
    return makeSepTermListSingle(
        acceptHead ? acceptHead(getSepTermListHead(arg)) : getSepTermListHead(arg));
  }
  if (isSepTermListMany(arg)) {
    return makeSepTermListMany(
        acceptHead ? acceptHead(getSepTermListHead(arg)) : getSepTermListHead(arg),
        acceptSep ? acceptSep(getSepTermListSep(arg)) : getSepTermListSep(arg),
        visitSepTermList(getSepTermListTail(arg), acceptHead, acceptSep));
  }
  ATabort("not a SepTermList: %t\n", arg);
  return (SepTermList)NULL;
}

/*}}}  */
/*{{{  SepStringList visitSepStringList(SepStringList arg, char* (*acceptHead)(char*), char* (*acceptSep)(char*)) */

SepStringList visitSepStringList(SepStringList arg, char* (*acceptHead)(char*), char* (*acceptSep)(char*))
{
  if (isSepStringListEmpty(arg)) {
    return makeSepStringListEmpty();
  }
  if (isSepStringListSingle(arg)) {
    return makeSepStringListSingle(
        acceptHead ? acceptHead(getSepStringListHead(arg)) : getSepStringListHead(arg));
  }
  if (isSepStringListMany(arg)) {
    return makeSepStringListMany(
        acceptHead ? acceptHead(getSepStringListHead(arg)) : getSepStringListHead(arg),
        acceptSep ? acceptSep(getSepStringListSep(arg)) : getSepStringListSep(arg),
        visitSepStringList(getSepStringListTail(arg), acceptHead, acceptSep));
  }
  ATabort("not a SepStringList: %t\n", arg);
  return (SepStringList)NULL;
}

/*}}}  */
/*{{{  Lexical visitLexical(Lexical arg, char* (*acceptString)(char*)) */

Lexical visitLexical(Lexical arg, char* (*acceptString)(char*))
{
  if (isLexicalDefault(arg)) {
    return makeLexicalDefault(
        acceptString ? acceptString(getLexicalString(arg)) : getLexicalString(arg));
  }
  ATabort("not a Lexical: %t\n", arg);
  return (Lexical)NULL;
}

/*}}}  */

/*}}}  */
